/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungDynamisch;
import de.bsvrz.sys.funclib.debug.Debug;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.CountryEnum;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.D2LogicalModel;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.Exchange;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.FaultSeverityEnum;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.HeaderInformation;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.InformationStatusEnum;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.InternationalIdentifier;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.TextDisplayAreaSettings;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.TextPage;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.Vms;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsFault;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsFaultEnum;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsMessage;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsMessageIndexVmsMessage;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsPublication;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsText;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsTextLine;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsTextLineIndexVmsTextLine;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsUnit;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsUnitRecordVersionedReference;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsUnitTableVersionedReference;
import eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen.VmsUnitVmsIndexVms;

/**
 * Klasse zur Konvertierung der dynamischen Inhalte von dWiSta-Anzeigen in das Datex-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class DWiStaAnzeigeInhalte implements DatexKonvertierungDynamisch {

  /** Versionsstring, falls Verknüpfung zu statischer Konvertierung nicht hergestellt werden kann. */
  private static final String FAIL_VERSION_STRING = "unknown";
  
  /** Schlüssel Standard-Intervall. */
  private static final String KEY_STANDARD_INTERVALL = "StandardIntervall";
  
  /** Schlüssel NationalIdentifier. */
  private static final String KEY_NATIONAL_IDENTIFIER = "National-Identifier";

  /** Schlüssel LocationGroupId. */
  private static final String KEY_VMS_TABLE_ID = "VmsTableId";

  /** Standard-Messintervall (in Sekunden). */
  private static final int STANDARD_INTERVALL = 60;

  /** Akzeptierter Datenverteiler-Objekttyp: Anzeige. */
  private static final String TYP_ANZEIGE = "anzeige";
  
  /** Akzeptierter Datenverteiler-Objekttyp: AnzeigeQuerschnitt. */
  private static final String TYP_QUERSCHNITT = "anzeigeQuerschnitt";

  /** Referenz auf Plugin-Definitions-Klasse. */
  private PluginExport plugin;

  /** Messintervall. */
  private float intervall;
  
  /** Id der LocationGroup. */
  private String vmsTableId;
  
  /** NationalIdentifier. */
  private String nationalIdentifier;
  
  /** Präfix für Log-Ausgaben. */
  private String logPrefix = "";
  
  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();

//  
//  private static final String D2L_VERSION = "2";
//  private String version = "unknown";
// 
  
  //public Map<String, String> zeichen;
  
  /** Liste edr Anzeigetafeln. */
  private List<AnzeigeTafel> anzeigetafelListe;
  
  /** Zuordnung AQ -> Daten. */
  private Map<String, ResultData> anzeigeQuerschnitteMap = new HashMap<String, ResultData>();

  /** Referenz auf statische Konvertierungsmethode. */
  private DWiStaVmsTable dWiStaVmsTable;
  
  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitionsinstanz.
   */
  
  public DWiStaAnzeigeInhalte(final PluginExport plugin) {
    this.plugin = plugin;
  }
  
  
  /**
   * Zugriff auf die PredefinedLocation Tabellenversion über die statische Konvertierungsmethode.
   *
   * @return Tabellenversion oder FAIL_VERSION_STRING, falls kein Zugriff möglich ist.
   */
  
  public String getVmsTableVersion() {
    String version = FAIL_VERSION_STRING;
    if (dWiStaVmsTable != null) {
      String letzteVerwendeteVersion = dWiStaVmsTable.getVmsTableVersion();
      if ((letzteVerwendeteVersion != null) && (!letzteVerwendeteVersion.isEmpty())) {
        version = letzteVerwendeteVersion;
      }
    }

    return version;
  }

  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
  	LOGGER.finest("DWiStaAnzeigeInhalte getDatexII()"); 
  
    D2LogicalModel d2l = new D2LogicalModel();   

    Map <String, ResultData> anzeigeObjekteMap = new HashMap<String, ResultData>();

    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) { 
        for (ResultData resultData : data.get(objekteAtgAsp)) {	
         if (objekteAtgAsp.getTyp().getName().equalsIgnoreCase(TYP_ANZEIGE)) {
      		  anzeigeObjekteMap.put(resultData.getObject().getPid(), resultData);
      	  }  else if (objekteAtgAsp.getTyp().getName().equalsIgnoreCase(TYP_QUERSCHNITT)) {
      		  anzeigeQuerschnitteMap.put(resultData.getObject().getPid(), resultData);
      	  }	      
        }      
    }
    VmsPublication payload = getVmsUnits(anzeigeObjekteMap, anzeigeQuerschnitteMap); 
    
    payload = getAndereVmsUnits(payload, anzeigeObjekteMap, anzeigeQuerschnitteMap); 
    
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setLang("DE");
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);

    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    payload.setPublicationCreator(internationalIdentifier);
    payload.setHeaderInformation(headerInformation);

    Date datum = new Date();
    XMLGregorianCalendar xmlGregorianCalendar = null;
  	xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(datum);
  
    payload.setPublicationTime(xmlGregorianCalendar);
    d2l.setPayloadPublication(payload);
    return d2l;
  }
  
  /**
   * Holt VMSUnits aus einer Map von AnzeigeQuerschnitten, die nicht im datenverteiler mit einer Anzeige verknüpft sind, 
   * ordnet ihr Anzeigen zu und fügt sie einer VMS-Publikation hinzu.
   * 
   * @param payload
   *          VmsPublication
   * @param anzeigeObjekteMap
   *           Map<String, ResultData> AnzeigeObjekte mit ihrer PID als Key
   * @param anzeigeQuerschnitteMap
   *           Map<String, ResultData> AnzeigeQuerschnitte mit ihrer PID als Key
   *  @return VmsPublication
   */
  
  private  VmsPublication getAndereVmsUnits(final VmsPublication payload, 
      final Map<String, ResultData> anzeigeObjekteMap, final Map<String, ResultData> anzeigeQuerschnitteMap) {
	  for (String key : anzeigeQuerschnitteMap.keySet()) {
		  try {
			  VmsUnit vmsUnit = new VmsUnit();
			  VmsUnitRecordVersionedReference vmsUnitRecordVersionedReference = new VmsUnitRecordVersionedReference();
			  vmsUnitRecordVersionedReference.setId(key);
			  vmsUnitRecordVersionedReference.setVersion(getVmsTableVersion());
			  VmsUnitTableVersionedReference vmsUnitTableVersionedReference = new VmsUnitTableVersionedReference();
			  vmsUnitTableVersionedReference.setId(vmsTableId);
			  vmsUnitTableVersionedReference.setVersion(getVmsTableVersion());
			  vmsUnit.setVmsUnitTableReference(vmsUnitTableVersionedReference);
			  vmsUnit.setVmsUnitReference(vmsUnitRecordVersionedReference);
			  int vmsCount = 0;
			  for (String anzeigeKey : anzeigeObjekteMap.keySet()) {
				  if (anzeigeKey.trim().toLowerCase().startsWith(key.trim().toLowerCase())) {
					  List<AnzeigeZeile> list = new ArrayList<AnzeigeZeile>();
					  AnzeigeZeile anzeigeZeile = new AnzeigeZeile(anzeigeKey, 1);
					  list.add(anzeigeZeile);
					  VmsUnitVmsIndexVms vmsUnitVmsIndexVms = new VmsUnitVmsIndexVms();
					  Vms vms = getVms(list, anzeigeObjekteMap);
					  vmsUnitVmsIndexVms.setVmsIndex(vmsCount);
					  vmsUnitVmsIndexVms.setVms(vms);
					  vmsUnit.getVms().add(vmsUnitVmsIndexVms);
					  vmsCount++;
				  }
			  }
			  if (vmsCount > 0) {
				  payload.getVmsUnits().add(vmsUnit);
			  }
		  } catch (Exception e) {
              LOGGER.warning("Daten nicht berücksichtigt.", e);
       }
	  }
	  return payload;
  }
  
  /**
   * Holt VMSUnits aus anzeigetafelListe, ordnet ihr Anzeigen zu und fügt sie einer VMS-Publikation hinzu.
   * 
   * @param anzeigeObjekteMap
   *           Map<String, ResultData> AnzeigeObjekte mit ihrer PID als Key
   * @param anzeigeQuerschnitteMap
   *           Map<String, ResultData> AnzeigeQuerschnitte mit ihrer PID als Key
   *  @return VmsPublication
   */

  private VmsPublication getVmsUnits(final Map<String, ResultData> anzeigeObjekteMap, 
      final Map<String, ResultData> anzeigeQuerschnitteMap) {  
	  VmsPublication payload = new VmsPublication();
	 
	  for (int i = 0; i < anzeigetafelListe.size(); i++) {
		  try {
			  AnzeigeTafel anzeigeTafel = anzeigetafelListe.get(i);
			  //LOGGER.info("anzeigeTafel.getId(): " + anzeigeTafel.getId());
			  if (anzeigeQuerschnitteMap.containsKey(anzeigeTafel.getId())) {
				  //ResultData resultData = anzeigeQuerschnitteMap.get(anzeigeTafel.getId());
				  String pid = anzeigeTafel.getId();
				  VmsUnit  vmsUnit = new  VmsUnit();
				  VmsUnitRecordVersionedReference vmsUnitRecordVersionedReference = new VmsUnitRecordVersionedReference();
				  vmsUnitRecordVersionedReference.setId(pid);
				  vmsUnitRecordVersionedReference.setVersion(getVmsTableVersion());
				  VmsUnitTableVersionedReference vmsUnitTableVersionedReference = new VmsUnitTableVersionedReference();
				  vmsUnitTableVersionedReference.setId(vmsTableId);
				  vmsUnitTableVersionedReference.setVersion(getVmsTableVersion());
				  vmsUnit.setVmsUnitTableReference(vmsUnitTableVersionedReference);
				  vmsUnit.setVmsUnitReference(vmsUnitRecordVersionedReference);
				  anzeigeQuerschnitteMap.remove(pid);
				  int vmsCount = 0;
				  if (anzeigeTafel.hasLinks()) {
					  VmsUnitVmsIndexVms vmsUnitVmsIndexVms = new VmsUnitVmsIndexVms();
					  Vms vms  = getVms(anzeigeTafel.getAnzeigenLinks(), anzeigeObjekteMap);
					  vmsUnitVmsIndexVms.setVmsIndex(vmsCount);
					  vmsUnitVmsIndexVms.setVms(vms);
					  vmsUnit.getVms().add(vmsUnitVmsIndexVms);
					  vmsCount++;
				  }
				  
				  if (anzeigeTafel.hasMitte()) {
					  VmsUnitVmsIndexVms vmsUnitVmsIndexVms = new VmsUnitVmsIndexVms();
					  Vms vms = getVms(anzeigeTafel.getAnzeigenMitte(), anzeigeObjekteMap);
					  vmsUnitVmsIndexVms.setVmsIndex(vmsCount);
					  vmsUnitVmsIndexVms.setVms(vms);
					  vmsUnit.getVms().add(vmsUnitVmsIndexVms);
					  vmsCount++;
				  }
				  if (anzeigeTafel.hasRechts()) {
					  VmsUnitVmsIndexVms vmsUnitVmsIndexVms = new VmsUnitVmsIndexVms();
					  Vms vms = getVms(anzeigeTafel.getAnzeigenRechts(), anzeigeObjekteMap);
					  vmsUnitVmsIndexVms.setVmsIndex(vmsCount);
					  vmsUnitVmsIndexVms.setVms(vms);
					  vmsUnit.getVms().add(vmsUnitVmsIndexVms);
					  vmsCount++;
				  }
				  payload.getVmsUnits().add(vmsUnit);
			  }
		  } catch (Exception e) {
              LOGGER.warning("Daten nicht berücksichtigt.", e);
		  }
		 
	  }  
	  return  payload;
  }
  
  /**
   * Generiert VMS aus einer Liste von AnzeigeZeilen.
   * 
   * @param anzeigeZeileList
   *           List<AnzeigeZeile> 
   * @param anzeigeObjekteMap
   *           Map<String, ResultData> Anzeigen
   *  @return Vms
   */
  
  private Vms getVms(final List<AnzeigeZeile> anzeigeZeileList, final Map<String, ResultData> anzeigeObjekteMap) {
	  //LOGGER.info("getVms()" + anzeigeObjekteMap.size());
	  Vms vms = new Vms();
	  TextDisplayAreaSettings textDisplayAreaSettings = new TextDisplayAreaSettings();
	  VmsMessage vmsMessage = new VmsMessage();
	  TextPage tp = new TextPage();
	  VmsText vmsText = new VmsText();
	  VmsMessageIndexVmsMessage vmsMessageIndexVmsMessage = new VmsMessageIndexVmsMessage();
	  int lineIndex = 0;
	  boolean textLanternsOn = false;
	  Collections.sort(anzeigeZeileList);
	  Calendar lastSetTime = Calendar.getInstance();
	  lastSetTime.setTimeInMillis(0); // Epoche
	  for (AnzeigeZeile anzeigeZeile: anzeigeZeileList) {
		  //LOGGER.info("anzeigePid: " + anzeigePid);
		  String anzeigePid = anzeigeZeile.getId();
		  if (anzeigeObjekteMap.containsKey(anzeigePid)) {
			  ResultData data = anzeigeObjekteMap.get(anzeigePid);
			  if (data.getData().getItem("Eigenschaft").getTextValue("Blinken").getText().equalsIgnoreCase("Ja")) {
				  textLanternsOn = true;
			  }
			  Calendar dataTimeSet = Calendar.getInstance();
			  dataTimeSet.setTimeInMillis(data.getDataTime());
			  if (dataTimeSet.after(lastSetTime)) {
			    lastSetTime = dataTimeSet;
			  }

			  if (data.getData().getTextValue("Status").getText().equalsIgnoreCase("OK")) {
				  vms.setVmsWorking(true);
			  } else {
			    XMLGregorianCalendar xmlDataZeit = WerkzeugSammlung.konvertiereDatum(dataTimeSet.getTime());
				  VmsFault vmsFault = new VmsFault();
				  //TODO Unterschiedliche Fehlerzustände berücksichtigen. Wo sollen die Zeiten her kommen?
				  vmsFault.setFaultCreationTime(xmlDataZeit);
				  vmsFault.setFaultLastUpdateTime(xmlDataZeit);
				  vmsFault.setFaultSeverity(FaultSeverityEnum.UNKNOWN);
				  vmsFault.setFaultIdentifier("undefiniert");
				  vmsFault.setFaultDescription("undefiniert");
				  vmsFault.setVmsFault(VmsFaultEnum.OTHER);
				  vms.getVmsFaults().add(vmsFault);
				  vms.setVmsWorking(false);
			  }
			  
			  VmsTextLineIndexVmsTextLine vmsTextLineIndexVmsTextLine = getVmsTextLine(data);
			  if (vmsTextLineIndexVmsTextLine.getVmsTextLine() != null) {
				  vmsTextLineIndexVmsTextLine.setLineIndex(lineIndex);
				  lineIndex++;
				  vmsText.getVmsTextLines().add(vmsTextLineIndexVmsTextLine);
			  }
			  textDisplayAreaSettings.setTextLanternsOn(textLanternsOn);
		  }
	  }
	  vms.setTextDisplayAreaSettings(textDisplayAreaSettings);
	  tp.setVmsText(vmsText);
	  tp.setPageNumber(0);
	  
    XMLGregorianCalendar xmlReferenzZeit = WerkzeugSammlung.konvertiereDatum(lastSetTime.getTime());
    vmsMessage.setTimeLastSet(xmlReferenzZeit);
    
	  vmsMessage.getTextPages().add(tp);
	  vmsMessageIndexVmsMessage.setMessageIndex(vms.getVmsMessages().size());
	  vmsMessageIndexVmsMessage.setVmsMessage(vmsMessage);
	  vms.getVmsMessages().add(vmsMessageIndexVmsMessage);
	  return vms;
  }
  
  /**
   * Generiert eine VMSTextzeile aus ResultData.
   * 
   * @param data
   *           ResultData
   *  @return VmsTextLineIndexVmsTextLine
   */
  
  private VmsTextLineIndexVmsTextLine getVmsTextLine(final ResultData data) {
	  VmsTextLineIndexVmsTextLine vmsTextLineIndexVmsTextLine = new VmsTextLineIndexVmsTextLine();
	  VmsTextLine vmsTextLine = new VmsTextLine();
	  TextArray anzeigenArr = data.getData().getItem("Eigenschaft").getItem("DynamischerWechseltext").
	      getTextArray("Textdefinition"); 
	  if (anzeigenArr.getLength() > 0) {
		 String anzeigeText = "";
		  for (int i = 0; i < anzeigenArr.getLength(); i++) {
			  String zeichenText = anzeigenArr.getText(i);
			  String zeichenPid = zeichenText.substring(0, zeichenText.indexOf("pid")).trim();
			  //LOGGER.info("zeichenPid: " + zeichenPid);
			  anzeigeText =  anzeigeText + dWiStaVmsTable.getZeichen().get(zeichenPid);
		  }
		 
		  vmsTextLine.setVmsTextLine(anzeigeText);
		  vmsTextLineIndexVmsTextLine.setVmsTextLine(vmsTextLine);
		  //hier String zusammensetzen.
	  }
	 
	  return vmsTextLineIndexVmsTextLine;
  }

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    String con = "eu.datex2.schema.dynamisch.anzeigetafeln.anzeigen";
    JAXBContext jbc = JAXBContext.newInstance(con);
    return jbc;
  }


  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    logPrefix = WerkzeugSammlung.erzeugeLogPrefix(plugin);
    
    intervall = STANDARD_INTERVALL;
    if (plugin.getKonfigurationsModul().hasData(KEY_STANDARD_INTERVALL)) {
      String standardIntervall = plugin.getKonfigurationsModul().getString(KEY_STANDARD_INTERVALL);
      try {
        intervall = Float.parseFloat(standardIntervall);
      } catch (Exception e) {
        LOGGER.warning(logPrefix + "StandardIntervall konnte nicht eingelesen werden (" 
            + standardIntervall + "), nutze Standard-Wert", e);
      }
    } else {
      LOGGER.error(logPrefix + "Das Feld '" + KEY_STANDARD_INTERVALL + "' wurde nicht in der Konfiguration angegeben, " 
                + "nutze Standard-Wert (" + intervall + " Sekunden)");      
    }
    
    if (!plugin.getKonfigurationsModul().hasData(KEY_VMS_TABLE_ID)) {
      LOGGER.error(logPrefix + "Das Feld '" + KEY_VMS_TABLE_ID + "' wurde nicht in der Konfiguration angegeben.");
      throw new ConfigurationException();
    } else {
      vmsTableId = plugin.getKonfigurationsModul().getString(KEY_VMS_TABLE_ID);
    }

    if (!plugin.getKonfigurationsModul().hasData(KEY_NATIONAL_IDENTIFIER)) {
      LOGGER.error(logPrefix + "Das Feld '" + KEY_NATIONAL_IDENTIFIER + "' wurde nicht in der Konfiguration angegeben.");
      throw new ConfigurationException();
    } else {
      nationalIdentifier = plugin.getKonfigurationsModul().getString(KEY_NATIONAL_IDENTIFIER);
    }

    try {
      this.dWiStaVmsTable = (DWiStaVmsTable) plugin.getDatexKonvertierungStatisch();
      this.anzeigetafelListe = dWiStaVmsTable.getAnzeigetafelListe();    
    } catch (Exception e) {
      LOGGER.warning(logPrefix + "Zugriff auf statische Konvertierungsdaten nicht möglich.", e);
      throw new ConfigurationException();
    }
  }

}
