/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import eu.datex2.schema.dynamisch.baustellen.AlertCDirection;
import eu.datex2.schema.dynamisch.baustellen.AlertCDirectionEnum;
import eu.datex2.schema.dynamisch.baustellen.AlertCLocation;
import eu.datex2.schema.dynamisch.baustellen.AlertCMethod4Linear;
import eu.datex2.schema.dynamisch.baustellen.AlertCMethod4PrimaryPointLocation;
import eu.datex2.schema.dynamisch.baustellen.AlertCMethod4SecondaryPointLocation;
import eu.datex2.schema.dynamisch.baustellen.OffsetDistance;

/**
 * Wrapper f&uuml;r {@link AlertCMethod4Linear}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public class Datex2OrtsreferenzenAlertCLinear extends AlertCMethod4Linear {
	private Array data;

	public Datex2OrtsreferenzenAlertCLinear(final Array data) {
		super();
		this.data = data;
	}

	public AlertCMethod4Linear getAlertCLinear() {
		final Data item = data.getItem(0);
		setAlertCLocationCountryCode(item.getItem("LänderCode").asTextValue().getText());
		setAlertCLocationTableNumber(item.getItem("TabellenNummer").asTextValue().getText());
		setAlertCLocationTableVersion(item.getItem("Version").asTextValue().getText());

		final AlertCMethod4SecondaryPointLocation alertCMethod4SecondaryPointLocation = new AlertCMethod4SecondaryPointLocation();
		AlertCLocation alertCLocation = new AlertCLocation();
		alertCLocation
				.setSpecificLocation(BigInteger.valueOf(item.getItem("VonLCLPunkt").asUnscaledValue().longValue()));
		alertCMethod4SecondaryPointLocation.setAlertCLocation(alertCLocation);
		OffsetDistance offsetDistance = new OffsetDistance();
		offsetDistance
				.setOffsetDistance(BigInteger.valueOf(item.getItem("VonLCLOffset").asUnscaledValue().longValue()));
		alertCMethod4SecondaryPointLocation.setOffsetDistance(offsetDistance);
		setAlertCMethod4SecondaryPointLocation(alertCMethod4SecondaryPointLocation);

		final AlertCMethod4PrimaryPointLocation alertCMethod4PrimaryPointLocation = new AlertCMethod4PrimaryPointLocation();
		alertCLocation = new AlertCLocation();
		alertCLocation
				.setSpecificLocation(BigInteger.valueOf(item.getItem("BisLCLPunkt").asUnscaledValue().longValue()));
		alertCMethod4PrimaryPointLocation.setAlertCLocation(alertCLocation);
		offsetDistance = new OffsetDistance();
		offsetDistance
				.setOffsetDistance(BigInteger.valueOf(item.getItem("BisLCLOffset").asUnscaledValue().longValue()));
		alertCMethod4PrimaryPointLocation.setOffsetDistance(offsetDistance);
		setAlertCMethod4PrimaryPointLocation(alertCMethod4PrimaryPointLocation);

		final AlertCDirection alertCDirection = new AlertCDirection();
		if ("positive".equals(item.getItem("VonLCLRichtung").asTextValue().getText())) {
			alertCDirection.setAlertCDirectionCoded(AlertCDirectionEnum.POSITIVE);
		} else {
			alertCDirection.setAlertCDirectionCoded(AlertCDirectionEnum.NEGATIVE);
		}
		setAlertCDirection(alertCDirection);

		return this;
	}
}
