package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.Collection;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungStatisch;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakte Klasse für gemeinsam genutzte Strukturen verschiedener Plugins.
 * 
 * @author TraffGo Road
 *
 */

public abstract class StandardKonvertierungStatisch implements DatexKonvertierungStatisch {
  /** Schlüssel PredefinedLocationVersion. */
  protected static final String KEY_PREDEFINED_LOCATION_VERSION = "PredefinedLocationVersion";
  /** Schlüssel LocationGroupId. */
  protected static final String KEY_LOCATION_GROUP_ID = "LocationGroupId";
  /** Standard-Ausgabe für Versionsstring. */
  protected static final String STANDARD_VERSION_STRING = "unknown";
  /** Schlüssel NationalIdentifier. */
  protected static final String KEY_NATIONAL_IDENTIFIER = "National-Identifier";
  /** Referenz auf Plugin-Definitions-Klasse. */
  protected PluginExport plugin;
  /** Logging. */
  protected static final Debug LOGGER = Debug.getLogger();
  /** NationalIdentifier. */
  protected String nationalIdentifier;
  /** Präfix für Log-Ausgaben. */
  protected String logPrefix = "";
  /** Kontext für JAXB. */
  protected JAXBContext jaxbContext;
  /** Version der PredefinedLocation. */
  protected String version = STANDARD_VERSION_STRING;

  /** Id der LocationGroup. */
  protected String locationGroupId;
  
  /**
   * Aktuell verwendete Version der PredefinedLocations.
   * 
   * @return Versionsstring
   */
  public String getPredefinedLocationVersion() {  
    return version;
  }
  
  /**
   * Die neue Version wird als Instanzvariable übernommen und persistiert. Die Methode gibt eine Warnung aus, 
   * wenn die Persistierung nicht erfolgen konnte.
   * 
   * @param versionFree Wenn true, wird die Version nicht übernommen und nicht persistiert.
   * @param newVersion String mit neuer Version
   */
  protected void persistiereVersion(final boolean versionFree, final String newVersion) {
    if (!versionFree) {
      this.version = newVersion;
      try {
        plugin.getPersistenzModul().putData(KEY_PREDEFINED_LOCATION_VERSION, version);
      } catch (Exception e) {
        LOGGER.warning(logPrefix + "PredefinedLocationVersion kann nicht persistiert werden.", e);
      }
    }
  }
  
  /**
   * Initialisierung von grundlegenden Feldern (NationalIdentifier, Log-Präfix).
   * 
   * @throws ConfigurationException Wird geworfen, wenn Pflichtfelder nicht eingelesen werden konnten.
   */

  protected void initialisiereBasisFelder() throws ConfigurationException {
    logPrefix = WerkzeugSammlung.erzeugeLogPrefix(plugin);

    nationalIdentifier = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_NATIONAL_IDENTIFIER);

    locationGroupId = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_LOCATION_GROUP_ID);

    if (plugin.getPersistenzModul().hasData(KEY_PREDEFINED_LOCATION_VERSION)) {
      this.version = plugin.getPersistenzModul().getString(KEY_PREDEFINED_LOCATION_VERSION);
    }
  }

  @Override
  public abstract Object getDatexII(Collection<ObjekteAtgAsp> objekteAtgAspListe, CreationOptions... options);
  
  @Override
  public abstract JAXBContext getJAXBContext() throws JAXBException;
  
  @Override
  public abstract void initialisiere(Collection<ObjekteAtgAsp> objects) throws ConfigurationException;
}