/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;
import java.math.BigInteger;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.CountryEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.D2LogicalModel;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.DirectionBearingValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.DirectionCompassEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.DirectionCompassValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.Exchange;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.FloatingPointMetreDistanceValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.HeaderInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.Humidity;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.HumidityInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.InformationStatusEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.IntegerMetreDistanceValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.InternationalIdentifier;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.KilogramsConcentrationValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MeasuredDataPublication;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MeasuredValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MeasurementSiteRecordVersionedReference;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MeasurementSiteTableVersionedReference;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MultilingualString;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MultilingualString.Values;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.MultilingualStringValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.PercentageValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.PrecipitationDetail;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.PrecipitationInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.PrecipitationIntensityValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.PrecipitationTypeEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.RoadSurfaceConditionInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.RoadSurfaceConditionMeasurements;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.SiteMeasurements;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.SiteMeasurementsIndexMeasuredValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.SpeedValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.Temperature;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.TemperatureInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.TemperatureValue;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.Visibility;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.VisibilityInformation;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.WeatherRelatedRoadConditionTypeEnum;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.Wind;
import eu.datex2.schema.dynamisch.messstellen.umfelddaten.WindInformation;

/**
 * Konvertierung von dynamischen Umfelddaten (Fahrbahntemperatur etc.) in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class UDRohdaten extends StandardKonvertierungDynamisch {

  /** Maximale Gradangabe bei Windrichtung. */
  private static final int MAX_GRAD = 360;

  /** Umrechnung km/h => m/s. */
	private static final float FACTOR_KMH_TO_MS = 3.6f;
	
	/** Umrechnung Löslichkeit in Prozent => Löslichkeit in kg/m^3. */
  private static final float FACTOR_SALZ_PERCENT_TO_KGPERM3 = 3.56f;
    
  /** Umrechnung mm => m. */
  private static final int FACTOR_METER_TO_MM = 1000;
  
  /** Grenzwert. */
  private static final float BIG_MINUS = -3001;
  
    /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_NORTH_NORTH_EAST = 11;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_NORTH_EAST = 33;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_EAST_NORTH_EAST = 55;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_EAST = 78;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_EAST_SOUTH_EAST = 101;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_SOUTH_EAST = 123;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_SOUTH_SOUTH_EAST = 146;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_SOUTH = 168;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_SOUTH_SOUTH_WEST = 191;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_SOUTH_WEST = 213;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_WEST_SOUTH_WEST = 236;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_WEST = 258;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_WEST_NORTH_WEST = 281;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_NORTH_WEST = 303;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_NORTH_NORTH_WEST = 326;

  /** Gradeinteilung Kompass. */
  private static final int DEG_BEGIN_NORTH = 349;
  
  /** Referenz auf die UDMeasurementsSite (zweite Referenz für Index etc.). */
  private UDMeasurementSite statischeKonvertierungUfd;

    /** Informationen über Fahrbahn. */
  protected static enum RoadSurfaceInformationEnum {
    /** Oberflächentemperatur. */ 
    ROAD_SURFACE_TEMPERATURE, 
    /** Gefriertemperatur. */ 
    PROTECTION_TEMPERATURE, 
    /** Restsalz. */ 
    DEICING_CONCENTRATION, 
    /** Schneehöhe. */ 
    DEPTH_OF_SNOW, 
    /** Wasserfilmdicke. */ 
    WATER_FILM_THICKNESS
  }
  
  /** Niederschlagsinformationen. */
  protected static enum PrecipitationInformationEnum {
    /** Art. */
    TYPE, 
    /** Intensität. */
    INTENSITY, 
    /** Menge. */
    DEPTH
  }
  
  /** Windinformationen. */
  protected static enum WindInformationEnum {
    /** Windgeschwindigkeit (im Mittel). */
    SPEED, 
    /** Spitzengeschwindigkeit. */
    MAXIMUM_SPEED, 
    /** Windrichtung. */
    DIRECTION_BEARING, 
    /** Windrichtung. */
    DIRECTION_COMPASS
  }
  
  /** Temperaturinformationen. */
  protected static enum TemperatureInformationEnum {
    /** Lufttemperatur. */
    AIR, 
    /** Taupunkt. */
    DEW_POINT
  }
  
  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse
   */
  public UDRohdaten(final PluginExport plugin) {
    this.plugin = plugin;
  }
  
  /**
   * Gebe den in der Methode {link UDRohdaten#getDatexII(Collection, CreationOptions...)}
   * verwendeten JAXBContext zurück. 
   * 
   * @throws JAXBException JAXBException
   * @see javax.xml.bind.JAXBContext#newInstance(String)
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Erzeugt einen Multilingualen String mit Ländercode DE.
   * 
   * @param simpleString
   *          Text für den deutschen Text des Multiligualen Strings
   * @return MultilingualString
   */

  protected static MultilingualString konvertiereString(final String simpleString) {
      MultilingualString multilingualString = new MultilingualString();
      Values values = new Values();
      MultilingualStringValue multilingualStringValue = new MultilingualStringValue();
      multilingualStringValue.setLang("DE");
      multilingualStringValue.setValue(simpleString);
      values.getValues().add(multilingualStringValue);
      multilingualString.setValues(values);
      return multilingualString;
    }
  
  /**
   * Zugriff auf das Mapping von Sensoren zu Messstellen (wie es im statischen Modul erzeugt wurde).
   * 
   * @return Mapping von SystemObject (DaV-Typ Umfelddatensensor) zu SystemObject (DaV-Typ Umfelddatenmessstelle)
   */
  
  public Map<SystemObject, SystemObject> holeMessstellenMapping() {
    return statischeKonvertierungUfd.getMappingUfds2Ufdms();
  }

  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
    String predefinedLocationVersion = holePredefinedLocationVersion();
    Map<SystemObject, SystemObject> ufds2Ufdms = holeMessstellenMapping();

    D2LogicalModel d2l = new D2LogicalModel();
    MeasuredDataPublication payload = new MeasuredDataPublication();
    
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setLang("DE");
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    MeasurementSiteTableVersionedReference measurementSiteTableVersionedReference = 
        new MeasurementSiteTableVersionedReference();
    measurementSiteTableVersionedReference.setVersion(predefinedLocationVersion);
    measurementSiteTableVersionedReference.setId(locationGroupId);
    payload.setMeasurementSiteTableReference(measurementSiteTableVersionedReference);
    payload.setPublicationCreator(internationalIdentifier);
    payload.setHeaderInformation(headerInformation);
    Date datum = new Date();
    XMLGregorianCalendar xmlGregorianCalendar = null;
    xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(datum);
    payload.setPublicationTime(xmlGregorianCalendar);

    Map<String, SiteMeasurements> usedUfdmsPids = new HashMap<>();

    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {	
      for (ResultData resultData : data.get(objekteAtgAsp)) {	
        if ((resultData != null) && (resultData.hasData())) {
          try {  
            SystemObject pidUfdms = ufds2Ufdms.get(resultData.getObject());
            if (pidUfdms != null) {
              SiteMeasurements siteMeasurements = usedUfdmsPids.get(pidUfdms.getPid());
              if (siteMeasurements == null) {
                siteMeasurements = new SiteMeasurements();
                usedUfdmsPids.put(pidUfdms.getPid(), siteMeasurements);
                payload.getSiteMeasurements().add(siteMeasurements);

                MeasurementSiteRecordVersionedReference measurementSiteRecordVersionedReference = 
                    new MeasurementSiteRecordVersionedReference();
                measurementSiteRecordVersionedReference.setVersion(predefinedLocationVersion);
                measurementSiteRecordVersionedReference.setId(pidUfdms.getPid());
                siteMeasurements.setMeasurementSiteReference(measurementSiteRecordVersionedReference);
                siteMeasurements.setMeasurementTimeDefault(xmlGregorianCalendar);
              }

              MeasuredValue measuredValue = getIndexedValue(resultData, siteMeasurements);
              
              switch (objekteAtgAsp.getTyp().getPid().toLowerCase()) {
              //FahrBahnOberFlächenZustand		    	
              case "typ.ufdsfahrbahnoberflächenzustand":		    		 
                setRoadSurfaceCondition(measuredValue, resultData, xmlGregorianCalendar);
                break;
                //FahrBahnOberFlächenTemperatur    	
              case "typ.ufdsfahrbahnoberflächentemperatur":    		 
                setRoadSurfaceInformation(measuredValue, resultData, 
                    RoadSurfaceInformationEnum.ROAD_SURFACE_TEMPERATURE, xmlGregorianCalendar);
                break;
                //GefrierTemperatur    	 
              case "typ.ufdsgefriertemperatur":    			        	 
                setRoadSurfaceInformation(measuredValue, resultData, 
                    RoadSurfaceInformationEnum.PROTECTION_TEMPERATURE, xmlGregorianCalendar); 
                break;
                //RestSalz    	 
              case "typ.ufdsrestsalz":    			        	 
                setRoadSurfaceInformation(measuredValue, resultData, 
                    RoadSurfaceInformationEnum.DEICING_CONCENTRATION, xmlGregorianCalendar);    
                break;
                //WasserFilmDicke    	 
              case "typ.ufdswasserfilmdicke":    		 	        	 
                setRoadSurfaceInformation(measuredValue, resultData, 
                    RoadSurfaceInformationEnum.WATER_FILM_THICKNESS, xmlGregorianCalendar);   
                break;
                //SchneeHöhe    	 
              case "typ.ufdsschneehöhe":    		 	        	 
                setRoadSurfaceInformation(measuredValue, resultData, 
                    RoadSurfaceInformationEnum.DEPTH_OF_SNOW, xmlGregorianCalendar);
                break;
                //LuftTemperatur   	 
              case "typ.ufdslufttemperatur":    		 	        	 
                setTemperatureInformation(measuredValue, resultData, 
                    TemperatureInformationEnum.AIR, xmlGregorianCalendar);
                break;
                //TaupunktTemperatur   	 
              case "typ.ufdstaupunkttemperatur":    		 	        	 
                setTemperatureInformation(measuredValue, resultData, 
                    TemperatureInformationEnum.DEW_POINT, xmlGregorianCalendar);   
                break;
                //NiederschlagsArt    	 
              case "typ.ufdsniederschlagsart":    		   	        	 
                setPrecipitationInformation(measuredValue, resultData, 
                    PrecipitationInformationEnum.TYPE, xmlGregorianCalendar);  
                break;
                //NiederschlagsIntensität    	 
              case "typ.ufdsniederschlagsintensität":    		    	        	 
                setPrecipitationInformation(measuredValue, resultData, 
                    PrecipitationInformationEnum.INTENSITY, xmlGregorianCalendar); 
                break;
                //NiederschlagsMenge   	 
              case "typ.ufdsniederschlagsmenge":    		   	        	 
                setPrecipitationInformation(measuredValue, resultData, 
                    PrecipitationInformationEnum.DEPTH, xmlGregorianCalendar);    
                break;
                //WindGeschwindigkeitMittelWert   	 
              case "typ.ufdswindgeschwindigkeitmittelwert":    		     	        	 
                setWindInformation(measuredValue, resultData, WindInformationEnum.SPEED, xmlGregorianCalendar);   
                break;
                //WindGeschwindigkeitSpitzenWert    	 
              case "typ.ufdswindgeschwindigkeitspitzenwert":    		    	        	 
                setWindInformation(measuredValue, resultData, 
                    WindInformationEnum.MAXIMUM_SPEED, xmlGregorianCalendar); 
                break;
                //WindRichtung    	 
              case "typ.ufdswindrichtung":    			        	 
                setWindInformation(measuredValue, resultData, 
                    WindInformationEnum.DIRECTION_BEARING, xmlGregorianCalendar); 
                setWindInformation(measuredValue, resultData, 
                    WindInformationEnum.DIRECTION_COMPASS, xmlGregorianCalendar); 
                break;
                //SichtWeite    	 
              case "typ.ufdssichtweite":    	    	        	 
                setVisibilityInformation(measuredValue, resultData, xmlGregorianCalendar);    	   
                break;
                //RelativeLuftFeuchte    	 
              case "typ.ufdsrelativeluftfeuchte":    		  	        	 
                setHumidityInformation(measuredValue, resultData, xmlGregorianCalendar);   
                break;

              default:
                break;
              } // end switch
            }
          } catch (Exception e) {
            LOGGER.warning("Daten nicht berücksichtigt.", e);
          }


        } // resultData != null
      } // end for 
    } // end for


    // Last minute change request: remove all data errors / empty entries

    for (SiteMeasurements siteMeasurement : payload.getSiteMeasurements()) {
      Iterator<SiteMeasurementsIndexMeasuredValue> i = siteMeasurement.getMeasuredValues().iterator();
      while (i.hasNext()) {
        SiteMeasurementsIndexMeasuredValue x = i.next();
        if (x.getMeasuredValue().getBasicData() == null) {
          i.remove();
        }
      }
    }

    Iterator<SiteMeasurements> i = payload.getSiteMeasurements().iterator();
    while (i.hasNext()) {
      SiteMeasurements x = i.next();
      if (x.getMeasuredValues().isEmpty()) {
        i.remove();
      }
    }
    
    if (payload.getSiteMeasurements().isEmpty()) {
      LOGGER.warning(logPrefix + "Kein valider Datensatz verfügbar, Publikation wird abgebrochen");
      return null;
    } else { 
      d2l.setPayloadPublication(payload);      
      return d2l;
    }
  }
  
  /**
   * Prüft, ob in der JAXB-Struktur SiteMeasurement bereits ein MeasuredValue für den Informationstyp des
   * in ResultData enthaltenen Sensors vorhanden ist. Wenn dies der Fall ist, wird die zugehörige
   * MeasuredValue-Struktur zurückgeliefert, ansonsten wird eine neue Instanz erzeugt und der
   * SiteMeasurement hinzugefügt.
   * 
   * @param resultData DaV-Daten zu Umfelddatensensor
   * @param siteMeasurements JAXB-Struktur der Umfelddatensensormessstelle
   * @return MeasuredValue JAXB-Struktur
   */

  private MeasuredValue getIndexedValue(final ResultData resultData, final SiteMeasurements siteMeasurements) {
    int idx = statischeKonvertierungUfd.getMeasurementSiteIndex(resultData.getObject().getPid());

    // Wurde Index bereits verwendet?

    for (SiteMeasurementsIndexMeasuredValue measuredValueProbe : siteMeasurements.getMeasuredValues()) {
      if (measuredValueProbe.getIndex() == idx) {
        return measuredValueProbe.getMeasuredValue();
      }
    }
    
    SiteMeasurementsIndexMeasuredValue siteMeasurementsIndexMeasuredValue = new SiteMeasurementsIndexMeasuredValue();
    MeasuredValue measuredValue = new MeasuredValue();      
    siteMeasurementsIndexMeasuredValue.setMeasuredValue(measuredValue);
    siteMeasurementsIndexMeasuredValue.setIndex(idx);
    siteMeasurements.getMeasuredValues().add(siteMeasurementsIndexMeasuredValue);
    
    return measuredValue;
  }
  
  
  /**
   * Setzt die Strassenoberflächenbeschaffenheit in DatexII in Abhängigkeit des Wertes aus dem DaV. 
   * 
   * @param measuredValue
   *          Referenz auf JAXB-Klasseninstanz für gemessene Werte (MeasuredValue)
   * @param data
   *          ResultData (vom Datenverteiler)
   * @param xmlGregorianCalendar
   *          XMLGregorianCalendar mit der aktuellen Zeit
   */
  
 protected void setRoadSurfaceCondition(final MeasuredValue measuredValue,
     final ResultData data,
     final XMLGregorianCalendar xmlGregorianCalendar) {

   RoadSurfaceConditionInformation roadSurfaceConditionInformation = 
       (RoadSurfaceConditionInformation) measuredValue.getBasicData();
   
   if (roadSurfaceConditionInformation == null) {
     roadSurfaceConditionInformation = new RoadSurfaceConditionInformation();
     measuredValue.setBasicData(roadSurfaceConditionInformation);
   }
   
   RoadSurfaceConditionMeasurements roadSurfaceConditionMeasurements = 
       roadSurfaceConditionInformation.getRoadSurfaceConditionMeasurements();
   
   if (roadSurfaceConditionMeasurements == null) {
     roadSurfaceConditionMeasurements = new RoadSurfaceConditionMeasurements();
     roadSurfaceConditionInformation.setRoadSurfaceConditionMeasurements(roadSurfaceConditionMeasurements);     
   }
   
	 roadSurfaceConditionInformation.setMeasurementOrCalculationTime(xmlGregorianCalendar);
	 String oberflaechenZustand  = WerkzeugSammlung.getStringValue(data, "FahrBahnOberFlächenZustand", "Wert");
	 
	 switch (oberflaechenZustand.trim().toLowerCase()) {
	 case "trocken":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.DRY);
		 break;
	 case "feucht":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.WET);
		 break;
	 case "nass":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.WET);
		 break;
	 case "gefrorenes wasser":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes()
		 .add(WeatherRelatedRoadConditionTypeEnum.WET_AND_ICY_ROAD);
		 break;
	 case "schnee/schneematsch":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes()
		 .add(WeatherRelatedRoadConditionTypeEnum.SNOW_ON_THE_ROAD);
		 break;
	 case "eis":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.ICE);
		 break;
	 case "raureif":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes()
		 .add(WeatherRelatedRoadConditionTypeEnum.FREEZING_OF_WET_ROADS);
		 break;
	 case "nicht ermittelbar":
	 case "fehlerhaft":
	 case "nicht ermittelbar/fehlerhaft":
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.OTHER);
		 break;
	 default:
		 roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().add(WeatherRelatedRoadConditionTypeEnum.OTHER);
		 break;
	 }	
   if ((roadSurfaceConditionMeasurements.getDeIcingConcentration() == null) 
       && (roadSurfaceConditionMeasurements.getDepthOfSnow() == null) 
       && (roadSurfaceConditionMeasurements.getProtectionTemperature() == null)
       && (roadSurfaceConditionMeasurements.getRoadSurfaceTemperature() == null)
       && (roadSurfaceConditionMeasurements.getWaterFilmThickness() == null)
       && (roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().isEmpty())) {
      measuredValue.setBasicData(null);
    }
 }
	  
	  
 /**
  * Setzt die Strassenoberflächeninformation in DatexII in Abhängigkeit des Wertes aus dem DaV.
  * 
  * @param measuredValue
  *          Referenz auf JAXB-Klasseninstanz für SiteMeasurements
  * @param data
  *          ResultData
  * @param information
  *          RoadSurfaceInformationEnum - steuert die Art der Information, die in DatexII gesetzt werden soll  
  * @param xmlGregorianCalendar
  *          XMLGregorianCalendar mit der aktuellen Zeit  
  */
  
  protected void setRoadSurfaceInformation(
      final MeasuredValue measuredValue,
      final ResultData data, 
      final RoadSurfaceInformationEnum information, final XMLGregorianCalendar xmlGregorianCalendar) {
	  
    RoadSurfaceConditionInformation roadSurfaceConditionInformation = 
        (RoadSurfaceConditionInformation) measuredValue.getBasicData();
    
    if (roadSurfaceConditionInformation == null) {
      roadSurfaceConditionInformation = new RoadSurfaceConditionInformation();
      measuredValue.setBasicData(roadSurfaceConditionInformation);
    }
    
    RoadSurfaceConditionMeasurements roadSurfaceConditionMeasurements = 
        roadSurfaceConditionInformation.getRoadSurfaceConditionMeasurements();
    
    if (roadSurfaceConditionMeasurements == null) {
      roadSurfaceConditionMeasurements = new RoadSurfaceConditionMeasurements();
      roadSurfaceConditionInformation.setRoadSurfaceConditionMeasurements(roadSurfaceConditionMeasurements);     
    }

    roadSurfaceConditionInformation.setMeasurementOrCalculationTime(xmlGregorianCalendar);
    
	  
	  switch (information) {
	  case ROAD_SURFACE_TEMPERATURE:
		  float oberflaechenTemperatur = WerkzeugSammlung.getFloatValue(data, "FahrBahnOberFlächenTemperatur", 
		      "Wert", BIG_MINUS, 2);
		  if (oberflaechenTemperatur != BIG_MINUS) {
		    TemperatureValue temperatureValue = new TemperatureValue();
			  temperatureValue.setTemperature(oberflaechenTemperatur);
	      roadSurfaceConditionMeasurements.setRoadSurfaceTemperature(temperatureValue);
		  }
		  break;
		  
	  case PROTECTION_TEMPERATURE:
		  float gefrierTemperatur = WerkzeugSammlung.getFloatValue(data, "GefrierTemperatur", "Wert", BIG_MINUS, 2);
		  if (gefrierTemperatur != BIG_MINUS) {
        TemperatureValue temperatureValue = new TemperatureValue();
			  temperatureValue.setTemperature(gefrierTemperatur);
	      roadSurfaceConditionMeasurements.setProtectionTemperature(temperatureValue);
      }
		  
		  break;
	  
	  case DEPTH_OF_SNOW:
		  float schneeHoehe = WerkzeugSammlung.getFloatValue(data, "SchneeHöhe", "Wert", SMALL_MINUS_FLOAT, 2);
		  if (schneeHoehe > SMALL_MINUS_FLOAT) {
		    FloatingPointMetreDistanceValue floatingPointMetreDistanceValue = new FloatingPointMetreDistanceValue();
			  floatingPointMetreDistanceValue.setFloatingPointMetreDistance(schneeHoehe);
	      roadSurfaceConditionMeasurements.setDepthOfSnow(floatingPointMetreDistanceValue);			  
      }
		  break;
		  
	  case WATER_FILM_THICKNESS:
		  float wasserFilmMm = WerkzeugSammlung.getFloatValue(data, "WasserFilmDicke", "Wert", SMALL_MINUS_FLOAT, 2);
		  if (wasserFilmMm > SMALL_MINUS_FLOAT) {
		    float wasserFilmMeter = WerkzeugSammlung.rundeZahl(wasserFilmMm / FACTOR_METER_TO_MM, 2);
		    FloatingPointMetreDistanceValue floatingPointMetreDistanceValue = new FloatingPointMetreDistanceValue();
			  floatingPointMetreDistanceValue.setFloatingPointMetreDistance(wasserFilmMeter);
	      roadSurfaceConditionMeasurements.setWaterFilmThickness(floatingPointMetreDistanceValue);
      }
		  break;
		  
	  case DEICING_CONCENTRATION:
		  float salz = WerkzeugSammlung.getFloatValue(data, "RestSalz", "Wert", SMALL_MINUS_FLOAT, 2);
		  if (salz > SMALL_MINUS_FLOAT) {
			  float salzInKilo = WerkzeugSammlung.rundeZahl(salz * FACTOR_SALZ_PERCENT_TO_KGPERM3, 2); 
		    KilogramsConcentrationValue kilogramsConcentrationValue = new KilogramsConcentrationValue();
			  kilogramsConcentrationValue.setKilogramsConcentration(salzInKilo);
	      roadSurfaceConditionMeasurements.setDeIcingConcentration(kilogramsConcentrationValue);
      }
		  break;
		  
		  default:
		    throw new IllegalArgumentException(logPrefix + "Unbekannter Sensor-Wert");
	  }
    if ((roadSurfaceConditionMeasurements.getDeIcingConcentration() == null) 
        && (roadSurfaceConditionMeasurements.getDepthOfSnow() == null) 
        && (roadSurfaceConditionMeasurements.getProtectionTemperature() == null)
        && (roadSurfaceConditionMeasurements.getRoadSurfaceTemperature() == null)
        && (roadSurfaceConditionInformation.getWeatherRelatedRoadConditionTypes().isEmpty())
        && (roadSurfaceConditionMeasurements.getWaterFilmThickness() == null)) {
       measuredValue.setBasicData(null);
     }
  }
  
  
  /**
   * Setzt die Niederschlagsinformation in DatexII in Abhängigkeit des Wertes aus dem DaV. 
   * 
   * @param measuredValue
   *          Referenz auf JAXB-Klasseninstanz für MeasuredValue
   * @param data
   *          ResultData
   * @param information
   *          PrecipitationInformationEnum -steuert die Art der Information, die in DatexII gesetzt werden soll 
   * @param xmlGregorianCalendar
   *          XMLGregorianCalendar mit der aktuellen Zeit      
   */

  protected void setPrecipitationInformation(final MeasuredValue measuredValue, final ResultData data, 
      final PrecipitationInformationEnum information, final XMLGregorianCalendar xmlGregorianCalendar) {

    PrecipitationInformation precipitationInformation = (PrecipitationInformation) measuredValue.getBasicData();

    if (precipitationInformation == null) {
      precipitationInformation = new PrecipitationInformation();
      measuredValue.setBasicData(precipitationInformation);
    }
    
    PrecipitationDetail precipitationDetail = precipitationInformation.getPrecipitationDetail();
        
    if (precipitationDetail == null) {
      precipitationDetail = new PrecipitationDetail();
      precipitationInformation.setPrecipitationDetail(precipitationDetail);
    }
    
    precipitationInformation.setMeasurementOrCalculationTime(xmlGregorianCalendar);
    boolean keinNiederschlag = false;
    switch (information) {

    case TYPE:
      String niederschlagTyp  = WerkzeugSammlung.getStringValue(data, "NiederschlagsArt", "Wert");

      switch (niederschlagTyp.trim().toLowerCase()) {
      case "kein niederschlag":
        keinNiederschlag = true;
        break;

      case "niederschlag aller art":
      case "leichter/mittlerer niederschlag":
      case "starker niederschlag":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.RAIN);
        break;

      case "regen":	
      case "regen wmo1":
      case "regen wmo2":
      case "regen wmo3":
      case "regen wmo4":
      case "regen wmo5":
      case "regen wmo6":
      case "regen wmo7":
      case "regen wmo8":
      case "regen wmo9":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.RAIN);
        break;

      case "sprühregen":
      case "sprühregen wmo1":
      case "sprühregen wmo2":
      case "sprühregen wmo3":
      case "sprühregen wmo4":
      case "sprühregen wmo5":
      case "sprühregen wmo6":
      case "sprühregen wmo7":
      case "sprühregen wmo8":
      case "sprühregen wmo9":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.DRIZZLE);
        break;

      case "schnee":
      case "schnee/gefrorener niederschlag":
      case "schneeg/gefrorener niederschlag":
      case "schnee wmo1":
      case "schnee wmo2":
      case "schnee wmo3":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.SNOW);
        break;

      case "graupel wmo1":
      case "graupel wmo2":
      case "graupel wmo3":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.SLEET);
        break;

      case "hagel wmo1":
      case "hagel wmo2":
      case "hagel wmo3":
        precipitationDetail.setPrecipitationType(PrecipitationTypeEnum.HAIL);
        break;

      case "nicht ermittelbar":
      case "fehlerhaft":
      case "nicht ermittelbar/fehlerhaft":
        keinNiederschlag = true;
        break;

      default:
        keinNiederschlag = true;
        break;
      } // end switch niederschlagTyp

      break;

    case INTENSITY: 
      float mmProStunde = WerkzeugSammlung.getFloatValue(data, "NiederschlagsIntensität", "Wert", SMALL_MINUS_FLOAT, 2);
      if (mmProStunde > SMALL_MINUS_FLOAT) {
        PrecipitationIntensityValue precipitationIntensityValue = new PrecipitationIntensityValue();
        precipitationIntensityValue.setMillimetresPerHourIntensity(mmProStunde);
        precipitationDetail.setPrecipitationIntensity(precipitationIntensityValue);
      }
      break;

    case DEPTH:
      float niederschlagsMenge = WerkzeugSammlung.getFloatValue(data, "NiederschlagsMenge", "Wert", SMALL_MINUS_FLOAT, 2);
      if (niederschlagsMenge > SMALL_MINUS_FLOAT) {
        FloatingPointMetreDistanceValue floatingPointMetreDistanceValue = new FloatingPointMetreDistanceValue();
        floatingPointMetreDistanceValue.setFloatingPointMetreDistance(niederschlagsMenge);
        precipitationDetail.setDepositionDepth(floatingPointMetreDistanceValue);
      }
      break;
    default:
      throw new IllegalArgumentException(logPrefix + "Unbekannter Sensor-Wert (Interner Fehler)");
    }

    if (keinNiederschlag) {
      precipitationInformation.setNoPrecipitation(true);
    } else {
      precipitationInformation.setPrecipitationDetail(precipitationDetail);
    }

    if ((precipitationDetail.getPrecipitationType() == null) 
        && (precipitationDetail.getDepositionDepth() == null) 
        && (precipitationDetail.getPrecipitationIntensity() == null)
        && (!precipitationInformation.isNoPrecipitation())) {
       measuredValue.setBasicData(null);
     }
  }

  
  /**
   * Setzt die Windinformation in DatexII in Abhängigkeit des Wertes aus dem DaV. 
   * 
   * @param measuredValue
   *          Referenz auf JAXB-Klasseninstanz für MeasuredValue
   * @param data
   *          ResultData
   * @param information
   *          WindInformationEnum - steuert die Art der Information, die in DatexII gesetzt werden soll       
   * @param xmlGregorianCalendar
   *          XMLGregorianCalendar mit der aktuellen Zeit
   */
  
 protected void setWindInformation(final MeasuredValue measuredValue, final ResultData data, 
     final WindInformationEnum information, final XMLGregorianCalendar xmlGregorianCalendar) {
	  
   WindInformation windInformation = (WindInformation) measuredValue.getBasicData();
   
   if (windInformation == null) {
     windInformation = new WindInformation();
     measuredValue.setBasicData(windInformation);
   }
   
	 Wind wind = windInformation.getWind();
	 
	 if (wind == null) {
	   wind = new Wind();
	   windInformation.setWind(wind);
	 }
	  
	  switch (information) {
	  		case SPEED:
	  			float speedMeterProSekunde = WerkzeugSammlung.getFloatValue(data, 
	  			    "WindGeschwindigkeitMittelWert", "Wert", SMALL_MINUS_FLOAT, 2);
	  			
	  			if (speedMeterProSekunde > SMALL_MINUS_FLOAT) {
	  				float speedKilometerProStunde = WerkzeugSammlung.rundeZahl(speedMeterProSekunde * FACTOR_KMH_TO_MS, 2);
	  				
	  		    SpeedValue speedValue = new SpeedValue();
	  				speedValue.setSpeed(speedKilometerProStunde);
	          wind.setWindSpeed(speedValue);
	  			}
	  		break;
	  		
	  		case MAXIMUM_SPEED: 
	  			float speedMeterProSekundeMax = WerkzeugSammlung.getFloatValue(data, 
	  			    "WindGeschwindigkeitSpitzenWert", "Wert", SMALL_MINUS_FLOAT, 2);
	  			if (speedMeterProSekundeMax > SMALL_MINUS_FLOAT) {
	  				float speedKilometerProStundeMax = 
	  				    WerkzeugSammlung.rundeZahl(speedMeterProSekundeMax * FACTOR_KMH_TO_MS, 2);
	  		    SpeedValue speedValue = new SpeedValue();
	  				speedValue.setSpeed(speedKilometerProStundeMax);
	          wind.setMaximumWindSpeed(speedValue);
          }
	  		break;
	  		
	  		case DIRECTION_BEARING:
	  			long windRichtung = WerkzeugSammlung.getLongValue(data, "WindRichtung", "Wert", SMALL_MINUS_LONG);
	  			if ((windRichtung >= 0) && (windRichtung <= MAX_GRAD)) {	
	  		    DirectionBearingValue directionBearingValue = new DirectionBearingValue();
	  				directionBearingValue.setDirectionBearing(BigInteger.valueOf(windRichtung));
	          wind.setWindDirectionBearing(directionBearingValue);
	  			}
	  			
	  		break;
	  		
	  		case DIRECTION_COMPASS:
	  	    DirectionCompassValue directionCompassValue = new DirectionCompassValue();
	  			long windRichtungLong = WerkzeugSammlung.getLongValue(data, "WindRichtung", "Wert", SMALL_MINUS_LONG);
	  			if (windRichtungLong <= SMALL_MINUS_LONG) {
	  			  directionCompassValue = null;
	  			} else {	
	  				directionCompassValue.setDirectionCompass(getDirectionCompassEnum(windRichtungLong));
	          wind.setWindDirectionCompass(directionCompassValue);
	  			}
	  			
		  		break;
        default:
          throw new IllegalArgumentException(logPrefix + "Unbekannter Sensor-Wert");
	  }

	   if ((wind.getMaximumWindSpeed() == null) 
	       && (wind.getWindDirectionBearing() == null) 
	       && (wind.getWindDirectionCompass() == null) 
	       && (wind.getWindSpeed() == null)) {
	      measuredValue.setBasicData(null);
	    }
}
 
 
 /**
  * Setzt die Temperaturinformation in DatexII in Abhängigkeit des Wertes aus dem DaV. 
  * 
  * @param measuredValue
  *          Referenz auf JAXB-Klasseninstanz für MeasuredValue
  * @param data
  *          ResultData
  * @param information
  *          TemperatureInformationEnum - steuert die Art der Information, die in DatexII gesetzt werden soll    
  * @param xmlGregorianCalendar
  *          XMLGregorianCalendar mit der aktuellen Zeit
  */
 
 protected void setTemperatureInformation(final MeasuredValue measuredValue, final ResultData data, 
     final TemperatureInformationEnum information, final XMLGregorianCalendar xmlGregorianCalendar) {
	  
   TemperatureInformation temperatureInformation = (TemperatureInformation) measuredValue.getBasicData();
   
   if (temperatureInformation == null) {
     temperatureInformation = new TemperatureInformation();
     measuredValue.setBasicData(temperatureInformation);
   }
  
   Temperature temperature = temperatureInformation.getTemperature();
   
   if (temperature == null) {
     temperature = new Temperature();
     temperatureInformation.setTemperature(temperature);
   }
   
	 switch (information) {
	  		case AIR:
	  			float airTemperature = WerkzeugSammlung.getFloatValue(data, "LuftTemperatur", "Wert", BIG_MINUS, 2);
	  			if (airTemperature != BIG_MINUS) {
	  			  TemperatureValue temperatureValue = new TemperatureValue();
	  				temperatureValue.setTemperature(airTemperature);
	          temperature.setAirTemperature(temperatureValue);
	  			}
	  		break;
	  		
	  		case DEW_POINT: 
	  			float dewPointTemperature = WerkzeugSammlung.getFloatValue(data, "TaupunktTemperatur", "Wert", BIG_MINUS, 2);
	  			if (dewPointTemperature != BIG_MINUS) {
            TemperatureValue temperatureValue = new TemperatureValue();
	  				temperatureValue.setTemperature(dewPointTemperature);
	          temperature.setDewPointTemperature(temperatureValue);
	  			}
	  		break;
	      default:
          throw new IllegalArgumentException(logPrefix + "Unbekannter Sensor-Wert");
	  }	  

   if ((temperature.getAirTemperature() == null) 
       && (temperature.getDewPointTemperature() == null) 
       && (temperature.getMaximumTemperature() == null) 
       && (temperature.getMinimumTemperature() == null)) {
     measuredValue.setBasicData(null);
   }

 }
 
 
 /**
  * Setzt die Sichtweiteninformation in DatexII in Abhängigkeit des Wertes aus dem DaV. 
  * 
  * @param measuredValue
  *          Referenz auf JAXB-Klasseninstanz für MeasuredValue
  * @param data
  *          ResultData 
  * @param xmlGregorianCalendar
  *          XMLGregorianCalendar mit der aktuellen Zeit
  */
 
 protected void setVisibilityInformation(final MeasuredValue measuredValue, final ResultData data, 
     final XMLGregorianCalendar xmlGregorianCalendar) {
	  
   VisibilityInformation visibilityInformation = (VisibilityInformation) measuredValue.getBasicData();
   if (visibilityInformation == null) {
     visibilityInformation = new VisibilityInformation();
     measuredValue.setBasicData(visibilityInformation);
   }
   
   Visibility visibility = visibilityInformation.getVisibility();
   
   if (visibility == null) {
     visibility = new Visibility();
     visibilityInformation.setVisibility(visibility);
   }
   
   visibilityInformation.setMeasurementOrCalculationTime(xmlGregorianCalendar);
   
   IntegerMetreDistanceValue integerMetreDistanceValue = new IntegerMetreDistanceValue();
	  //TODO Prüfen, welche Einheit zurückkommt -> mom. überall keine Daten
   long sichtWeite = WerkzeugSammlung.getLongValue(data, "SichtWeite", "Wert", SMALL_MINUS_LONG);
	 if (sichtWeite > SMALL_MINUS_LONG) {
     integerMetreDistanceValue.setIntegerMetreDistance(BigInteger.valueOf(sichtWeite));

	   visibility.setMinimumVisibilityDistance(integerMetreDistanceValue);		
	 }

   if (visibility.getMinimumVisibilityDistance() == null) {
	   measuredValue.setBasicData(null);
	 }
 }
 
 
 /**
  * Setzt die Luftfeuchtigkeitsinformation in DatexII in Abhängigkeit des Wertes aus dem DaV. 
  * 
  * @param measuredValue
  *          Referenz auf JAXB-Klasseninstanz für MeasuredValue
  * @param data
  *          ResultData   
  * @param xmlGregorianCalendar
  *          XMLGregorianCalendar mit der aktuellen Zeit
  */
 
 protected void setHumidityInformation(final MeasuredValue measuredValue, final ResultData data, 
     final XMLGregorianCalendar xmlGregorianCalendar) {
	  
   HumidityInformation humidityInformation = (HumidityInformation) measuredValue.getBasicData();
   if (humidityInformation == null) {
     humidityInformation = new HumidityInformation();
     measuredValue.setBasicData(humidityInformation);
   }
   
   Humidity humidity = humidityInformation.getHumidity();
   
   if (humidity == null) {
     humidity = new Humidity();
     humidityInformation.setHumidity(humidity);
   }

   humidityInformation.setMeasurementOrCalculationTime(xmlGregorianCalendar);
       
	 PercentageValue percentageValue = new PercentageValue();
	 float luftFeuchte = WerkzeugSammlung.getFloatValue(data, "RelativeLuftFeuchte", "Wert", SMALL_MINUS_FLOAT, 2);
	 if (luftFeuchte > SMALL_MINUS_FLOAT) {
		 percentageValue.setPercentage(luftFeuchte);
  	 humidity.setRelativeHumidity(percentageValue);		
	 }
	 
	 if (humidity.getRelativeHumidity() == null) {
	   measuredValue.setBasicData(null);
	 }
}
 
 /**
  * Gibt die Himmelsrichtung zurück.
  *  
  * @param grad [0;360] Richtung in Grad.
  * @return Himmelsrichtung in DATEX-II Struktur
  */
 protected static DirectionCompassEnum getDirectionCompassEnum(final Long grad) {
	 if (grad >= DEG_BEGIN_NORTH_NORTH_EAST && grad < DEG_BEGIN_NORTH_EAST) {
		 return DirectionCompassEnum.NORTH_NORTH_EAST;
	 } else if (grad >= DEG_BEGIN_NORTH_EAST && grad < DEG_BEGIN_EAST_NORTH_EAST) {
		 return DirectionCompassEnum.NORTH_EAST;
	 } else if (grad >= DEG_BEGIN_EAST_NORTH_EAST && grad < DEG_BEGIN_EAST) {
		 return DirectionCompassEnum.EAST_NORTH_EAST;
	 } else if (grad >= DEG_BEGIN_EAST && grad < DEG_BEGIN_EAST_SOUTH_EAST) {
		 return DirectionCompassEnum.EAST;
   } else if (grad >= DEG_BEGIN_EAST_SOUTH_EAST && grad < DEG_BEGIN_SOUTH_EAST) {
		 return DirectionCompassEnum.EAST_SOUTH_EAST;
   } else if (grad >= DEG_BEGIN_SOUTH_EAST && grad < DEG_BEGIN_SOUTH_SOUTH_EAST) {
		 return DirectionCompassEnum.SOUTH_EAST;
   } else if (grad >= DEG_BEGIN_SOUTH_SOUTH_EAST && grad < DEG_BEGIN_SOUTH) {
		 return DirectionCompassEnum.SOUTH_SOUTH_EAST;
   } else if (grad >= DEG_BEGIN_SOUTH && grad < DEG_BEGIN_SOUTH_SOUTH_WEST) {
		 return DirectionCompassEnum.SOUTH;
   } else if (grad >= DEG_BEGIN_SOUTH_SOUTH_WEST && grad < DEG_BEGIN_SOUTH_WEST) {
		 return DirectionCompassEnum.SOUTH_SOUTH_WEST;
   } else if (grad >= DEG_BEGIN_SOUTH_WEST && grad < DEG_BEGIN_WEST_SOUTH_WEST) {
		 return DirectionCompassEnum.SOUTH_WEST;
   } else if (grad >= DEG_BEGIN_WEST_SOUTH_WEST && grad < DEG_BEGIN_WEST) {
		 return DirectionCompassEnum.WEST_SOUTH_WEST;
   } else if (grad >= DEG_BEGIN_WEST && grad < DEG_BEGIN_WEST_NORTH_WEST) {
		 return DirectionCompassEnum.WEST;
   } else if (grad >= DEG_BEGIN_WEST_NORTH_WEST && grad < DEG_BEGIN_NORTH_WEST) {
		 return DirectionCompassEnum.WEST_NORTH_WEST;
   } else if (grad >= DEG_BEGIN_NORTH_WEST && grad < DEG_BEGIN_NORTH_NORTH_WEST) {
		 return DirectionCompassEnum.NORTH_WEST;
   } else if (grad >= DEG_BEGIN_NORTH_NORTH_WEST && grad < DEG_BEGIN_NORTH) {
		 return DirectionCompassEnum.NORTH_NORTH_WEST;
	 } else {
		 return DirectionCompassEnum.NORTH;
	 }
 }


  /** {@inheritDoc} 
   */ 
  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {   
    initialisiereBasisFelder();
    statischeKonvertierungUfd = (UDMeasurementSite) statischeKonvertierung;
  }
  
}
