/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente;

import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Datex2BaustelleException;
import eu.datex2.schema.dynamisch.baustellen.SeverityEnum;

/**
 * Umsetzung Datk <code>att.mdmBisVerkehrsBehinderung</code> in Datex2
 * <code>SeverityEnum</code>.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public enum Datex2SeverityEnum {

	/** Perceived by supplier as being of an unknown level. */
	Unbekannt(SeverityEnum.UNKNOWN, 0),

	/** Perceived by supplier as having a severity rating of none. */
	None(SeverityEnum.NONE, 1),

	/** Perceived by supplier as being of a low level. */
	Low(SeverityEnum.LOW, 2),

	/** Perceived by supplier as being of a medium level. */
	Medium(SeverityEnum.MEDIUM, 3),

	/** Perceived by supplier as being of a high level. */
	High(SeverityEnum.HIGH, 4);

	/**
	 * Gibt eine passende Severity-Instanz zur&uuml;ck.
	 *
	 * @param davwert
	 *            DAV-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static SeverityEnum getInstance(final int davwert) throws Datex2BaustelleException {
		for (Datex2SeverityEnum adm : Datex2SeverityEnum.values()) {
			if (adm.getDatexWert() != null && adm.getId() == davwert) {
				return adm.getDatexWert();
			}
		}
		throw new Datex2BaustelleException("Keine Datex2-'Severity' zu DAV-Wert=" + davwert + " definiert.");
	}

	/**
	 * textueller Wert.
	 */
	private final SeverityEnum datexWert;

	/**
	 * numerischer Wert.
	 */
	private int david;

	/**
	 * Privater Konstruktor.
	 *
	 * @param value
	 *            textueller Wert
	 * @param id
	 *            numerischer Wert
	 */
	Datex2SeverityEnum(final SeverityEnum value, final int id) {
		this.datexWert = value;
		this.david = id;
	}

	/**
	 * Gibt den numerischen Wert zur&uuml;ck.
	 *
	 * @return numerischer Wert der Himmelsrichtung
	 */
	public int getId() {
		return david;
	}

	private SeverityEnum getDatexWert() {
		return datexWert;
	}
}
